clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define parameters

prm.dt = 0.01;        % time step (sec)
prm.m = 1;            % mass (kg)
prm.b = 0;            % damping (N/sec)
prm.tau = 40;         % time constant (msec)
prm.c = 0.5;          % control-dependent noise

prm.r = 0.00001;      % control signal penalty
prm.v = 0.2;          % endpoint velocity penalty
prm.f = 0.02;         % endpoint force penalty

prm.pos = 0.5*0.02;   % position noise
prm.vel = 0.5*0.2;    % velocity noise
prm.frc = 0.5*1.0;    % force noise

N = 30;               % duration in number of time steps
T = 0.1;              % target distance


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute system dynamics and cost matrices

dtt = prm.dt/(prm.tau/1000);

A = zeros(5,5);
A(1,1) = 1;
A(1,2) = prm.dt;
A(2,2) = 1-prm.dt*prm.b/prm.m;
A(2,3) = prm.dt/prm.m;
A(3,3) = 1-dtt;
A(3,4) = dtt;
A(4,4) = 1-dtt;
A(5,5) = 1;

B = zeros(5,1);
B(4,1) = dtt;

C = prm.c;
C0 = 0;


H = zeros(3,5);
H(1:3,1:3) = eye(3);

D = 0;
D0 = diag([prm.pos prm.vel prm.frc]);

E0 = 0;

R = prm.r/N;
   
Q = zeros(5,5,N);

d = zeros(3,5);
d(1,1) = 1;
d(1,5) = -1;
d(2,2) = prm.v;
d(3,3) = prm.f;
Q(:,:,N) = d'*d;

X1 = zeros(5,1);
X1(5) = T;
S1 = zeros(5,5);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% optimize, plot simulated trajectories

[K,L,Cost,Xa,XSim,CostSim] = ...
   kalman_lqg( A,B,C,C0, H,0,D0, 0, Q,R, X1,S1, 10 );
clf;
plot(squeeze(XSim(1,:,:))','r'); hold on;
plot(Xa(1,:),'k','linewidth',2);
xlabel('time step');
ylabel('position');